/* Copyright (c) 1994 Peter Apian-Bennewitz */

#ifdef PROTO
#include  <stdlib.h>
#endif
#include  <unistd.h>
#include  <stdio.h>
#include  <math.h>
#include  "color.h"
#include  "resolu.h"

static	char rcsid[]="$Header: /usr/people/apian/radiance/gen/genpic/RCS/genpic.c,v 1.9 95/05/18 13:52:48 apian Exp Locker: apian $";

int	verbose_flag=0;


/*----------------------------------------------------------------------------------------------------*/
#ifdef PROTO
void badopt( char *s) 
#else
void badopt( s ) 
char *s;
#endif
{
	fprintf(stderr,"genpic: bad option at \"%s\"\n",s);
	exit(1);
}
/*----------------------------------------------------------------------------------------------------*/

#ifdef PROTO
void check_file( char *file, int *xmax, int *ymax )	/* return size of radiance picture */
#else
void check_file( file, xmax, ymax )
char *file; int *xmax; int *ymax;
#endif
{
	FILE *fid;

	if( !(fid=fopen(file,"r") ) ) {
	  fprintf(stderr,"genpic: can't open picture file %s\n",file);
	  exit(1);
	}
	if( checkheader(fid, COLRFMT, NULL) < 0 || fgetresolu(xmax, ymax, fid) < 0){
	  fprintf(stderr,"genpic: can't read header in picture file %s\n",file);
	  exit(1);
	}
	fclose (fid);
}
/*----------------------------------------------------------------------------------------------------*/

main(argc, argv)
int  argc;
char  *argv[];
{
	char *picname;
	float lsize=1.0, aspect;
	int xmax, ymax;
	float ysize, zsize, minsize;				/* units in [m] */
	float frame_border=0.01, frame_thickness=0.02;		/* frame geometry */
	float picoffset=0.001, glasoffset=0.001;		/* distance between picture layers */
	float aspect_ratio= 0.0;
	float glow_intens=0.0, glow_radius= -1.0;
	int uniqid;
	register int i;
	char modifier[1024];
	int no_mat_flag=0, no_frame_flag=0;

/*
	material/color for frame
	wood-frame ?
	glass n
*/

	for (i = 1; i < argc && argv[i][0] == '-'; i++) {	/* option city */
		if (!strcmp(argv[i], "-help")) {
			fprintf(stderr, "usage: genpic [options] [-a]|[pictfile]\n");
			 fprintf(stderr,"       generate Radiance geometry and texure for a framed picture from picture file\n");
			 fprintf(stderr,"       -s       length of larger side\n");
			 fprintf(stderr,"       -fb      frame_border\n");
			 fprintf(stderr,"       -ft      frame_thickness\n");
			 fprintf(stderr,"       -g i r   generate glowing picture: i [cd/m2] intensity, r radius\n");
			 fprintf(stderr,"       -a       synthetic aspect_ratio \n");
			 fprintf(stderr,"       -n	 generate no frame\n");
			 fprintf(stderr,"       -m	 generate no material\n");
			 fprintf(stderr,"       pictfile Radiance picture file\n");
			 fprintf(stderr,"       either -a  or a pictfile must be given\n");
			exit(0);
		}

		switch (argv[i][1]) {

		case 'f':		
			switch (argv[i][2]) {
			case 't':			/* -ft frame_thickness */
				++i;
				frame_thickness= fabs(atof( argv[i] ));
			break;
			case 'b':			/* -ft frame_border */
				++i;
				frame_border= fabs(atof( argv[i] ));
			break;
			default: badopt(argv[i]);
			}
		break;

		case 'v':				/* -n no frame */
			verbose_flag=1;
		break;
		case 'n':				/* -n no frame */
			no_frame_flag=1;
		break;
		case 'm':				/* -m no material */
			no_mat_flag=1;
		break;
		case 's':				/* -s larger size */
			++i;
			lsize= atof( argv[i] );
		break;

		case 'a':				/* -a aspect_ratio */
			++i;
			aspect_ratio= fabs(atof( argv[i] ));
		break;
		case 'g':			/* -g i r  glow */
			++i;
			glow_intens= fabs(atof( argv[i] ));
			++i;
			glow_radius= atof( argv[i] );
		break;

		default: badopt(argv[i]);
		}
	}

	if( i!=argc-1 ) {
	  if( aspect_ratio == 0.0 ) {
	    fprintf(stderr,"picture file missing (try \"genpic -help\")\n");
	    exit(1);
	  }
	} else {
	  picname= argv[i];
	}

#ifdef MSDOS
	uniqid=0;
#else
	if( no_mat_flag )
	  uniqid=0;
	else	
	  uniqid=getpid();
#endif


	picoffset += frame_thickness;		/* convert offsets to absolute values */
	glasoffset += picoffset;

	if( aspect_ratio == 0.0 ) {
	  check_file( picname, &xmax, &ymax );	/* get picture */
	  if( verbose_flag ) fprintf(stderr,"genpic: %s %dx%d\n",picname,xmax,ymax );
	  aspect= (float)xmax/ymax;
	} else {
	  aspect=aspect_ratio;
	}

	if( aspect < 1.0 ) {		/* hochkant */
	  minsize = ysize= aspect * lsize; 
	  zsize= lsize; 
	} else {			/* Breitwand */
	  ysize= lsize; 
	  minsize = zsize= (1.0/aspect) * lsize; 
	}
	
	fprintf( stdout, "# " );
	for( i=0; i<argc; i++) fprintf( stdout,"%s ",argv[i] );
	fprintf( stdout, "\n# %s\n",rcsid );

	if( aspect_ratio==0 ) {
	  fprintf( stdout, "void colorpict genpic_pic_%d\n",uniqid );
	  fprintf( stdout, "15 clip_r clip_g clip_b %s picture.cal pic_v pic_u -ry -90 -s %g -t 0 %g %g\n",
		picname, minsize, -0.5*ysize, -zsize );
	  fprintf( stdout, "0\n0\n" );
	  sprintf( modifier,"genpic_pic_%d",uniqid);
	} else {
	  sprintf( modifier,"void");
	}

	if( glow_intens == 0.0 ) {
	  fprintf( stdout, "%s plastic genpic_picp_%d\n", modifier, uniqid, uniqid );
	  fprintf( stdout, "0\n0\n" );
	  fprintf( stdout, "5 1 1 1 0 0\n" );
	} else {
	  fprintf( stdout, "%s glow genpic_picp_%d\n", modifier, uniqid, uniqid );
	  fprintf( stdout, "0\n0\n" );
	  fprintf( stdout, "4 %g %g %g %g\n",glow_intens,glow_intens,glow_intens,glow_radius );
	}

	fprintf( stdout, "void glass genpic_glass_%d\n",uniqid );
	fprintf( stdout, "0\n0\n" );
	fprintf( stdout, "4 .88 .88 .88 1.2\n" );

	if( !no_mat_flag ) {
	  fprintf( stdout, "void metal genpic_frame_%d\n",uniqid );
	  fprintf( stdout, "0\n0\n" );
	  fprintf( stdout, "5 .5 .5 .5  .9 0\n" );
	}

	fprintf( stdout, "genpic_picp_%d polygon pol_bild1\n", uniqid );  /* picture mapped in yz plane */
	fprintf( stdout, "0\n0\n12 " );
	fprintf( stdout, "%g %g %g ", picoffset, ysize*0.5, -zsize );
	fprintf( stdout, "%g %g  0 ", picoffset,  ysize*0.5 );
	fprintf( stdout, "%g %g  0 ", picoffset, -ysize*0.5 );
	fprintf( stdout, "%g %g %g\n ", picoffset, -ysize*0.5, -zsize );

	fprintf( stdout, "genpic_glass_%d polygon pol_bild2\n", uniqid );  /* picture mapped in yz plane */
	fprintf( stdout, "0\n0\n12 " );
	fprintf( stdout, "%g %g %g ", glasoffset, ysize*0.5, -zsize );
	fprintf( stdout, "%g %g  0 ", glasoffset,  ysize*0.5 );
	fprintf( stdout, "%g %g  0 ", glasoffset, -ysize*0.5 );
	fprintf( stdout, "%g %g %g\n", glasoffset, -ysize*0.5, -zsize );

	if( !no_frame_flag ) 
	  fprintf( stdout, "!genbox genpic_frame_%d rahmen %g %g %g -b 0.005 | xform -t 0 %g %g\n",
			uniqid, 
			frame_thickness, ysize+2.0*frame_border, zsize+2.0*frame_border,
			-frame_border-0.5*ysize, -frame_border-zsize);
	exit(0);
}
/*----------------------------------------------------------------------------------------------------*/
